//Homework 8
//Carlos Rios
//Collaboration Statement: I worked on this homework problem using outside internet sources.

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* Creates the user interface for the game. The user 
* interface contains a top panel that shows the number 
* of iterations of the current run. A middle panel 
* contains a button to start/stop the game, a button to
* clear the cells, a button to single step the game and
* a text field to change the delay interval when 
* running with a timer. The bottom panel graphically 
* dispays all the cells in the game.
*
* @author Carlos Rios
* @version 1.0
*/

public class GameOfLifePanel extends JPanel
{	
	/** 
	* iterationNumber is the current iteration of the 
	*  game 
	*/
 	int iterationNumber=0;
	
	/** 
	* intervalsize contains the delay between iterations 
	*  when the game runs automatically 
	*/
	int intervalsize=100;
	
	/** 
	* squares is an array containing the squares that 
	*  describe the game state 
	*/
	Square[] squares;
	
	/** 
	* gameCellModel contains the current state of the game
	*  and can also create an iteration 
	*/
	CellModel gameCellModel=new CellModel();
	
	/** 
	* intervalTimer is the timer instance used when the 
	*	game runs automatically 
	*/
	Timer intervalTimer;
	
	/** 
	* interval is the input field for the value of the 
	*  delay between iterations 
	*/
	JTextField interval;
	
	/**
	* intervalStatusLabel displays the current value of 
	*  the delay between iterations
	*/
	JLabel intervalStatusLabel;
	
	/** 
	* iterationLabel displays the cumulative number of  
	*	iterations when the game runs automatically 
	*/
	JLabel iterationLabel;

   /**
   * GameOfLifePanel constructor initializes the user 
	*  interface
   */		
   public GameOfLifePanel()
   {	
	
		
		// initialize the interval timer 
		intervalTimer = new Timer(100,new TimerListener());
		intervalTimer.stop();
	
		// Set up top subpanel
      JPanel subPanel1 = new JPanel();
      subPanel1.setPreferredSize (new Dimension(750, 50));
      subPanel1.setBackground (Color.green);
      iterationLabel = new JLabel (" Iteration Number:  "+ iterationNumber);
      subPanel1.add (iterationLabel);
		
		// Setup panel to hold text field and Jlabel that asks for the interval
		JPanel intervalPanel= new JPanel();
		intervalPanel.setLayout( new BoxLayout(intervalPanel,BoxLayout.Y_AXIS));
		
		 interval = new JTextField("100",5);	
		interval.addActionListener(new IntervalListener());
		
		JLabel intervalInputLabel=new JLabel(" Input the delay interval below.");
		intervalStatusLabel=new JLabel(" The current interval is: "+interval.getText());
		intervalsize=Integer.parseInt( interval.getText());
		intervalPanel.add(intervalInputLabel);
		intervalPanel.add(interval);
		intervalPanel.add(intervalStatusLabel);
		
		

      // Set up middle subpanel
		// add buttons and intervalPanel
      JPanel subPanel2 = new JPanel();
      subPanel2.setPreferredSize (new Dimension(750, 100));
		JButton runStop= new JButton(" Run/Stop ");
		runStop.addActionListener(new StartStopListener());
		JButton step = new JButton(" Step ");
		step.addActionListener(new StepListener());
		JButton clear= new JButton(" Clear ");
		clear.addActionListener(new ClearListener());
		subPanel2.add(runStop);
		subPanel2.add(step);
		subPanel2.add(clear);
		subPanel2.add(intervalPanel);

		
		// Set up the bottom subpanel
		// the bottom subpanel is the game dispaly areas and consists
		// of an array of squares
		JPanel subPanel3 = new JPanel();
		subPanel3.setLayout( new GridLayout(20,20));	
		squares= new Square[400];	
		for (int i=0; i<400 ;i++)
		{
			squares[i]=new Square();
			squares[i].addMouseListener(new SquareListener());
		   subPanel3.add(squares[i]);
		}
      subPanel3.setPreferredSize (new Dimension(750,500));


		

    // setup primary panel (e.g. this panel)
		setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));
      this.setBackground (Color.white);
      this.add (subPanel1);
     this.add (subPanel2);
		this.add (subPanel3);
		
	// 
	
		
	}

  

/**
* SquareListener is an internal class that waits for a 
*  mouse click in one of the squares. An instance of 
*  this class is associated with each square in the 
*  array of squares.
* @author Victoria Lemay
* @version 1.0
*/

		private class SquareListener extends MouseAdapter
		{
			public void mouseClicked(MouseEvent event)
			{ for(int j=0; j<400 ;j++)
				{					
				 	if(event.getComponent()==squares[j])
					{
						if(squares[j].getState())
						{ squares[j].setState(false);}
						else
						{squares[j].setState(true);}
					}
				}
				
					repaint();
			}

		}

/**
* StepListener is an internal class	that listens to the 
*  step button. An event causes the game to iterate once.
* @author Victoria Lemay
* @version 1.0
*/
		private class StepListener implements ActionListener
		{	
			
			public void actionPerformed(ActionEvent event)
			{
						
			 gameCellModel.initializeModel(squares);
			 Square[] squaresAfterStep=gameCellModel.step();
				for(int i=0;i<400;i++)
				{
				squares[i].setState(squaresAfterStep[i].getState());
				}
				iterationNumber+=1;
				iterationLabel.setText(" Iteration Number:  "+Integer.toString(iterationNumber));
				repaint();

		
			}
		}	
		

/**
* ClearListener is an internal class that listens to the
*   clear button. Clears all of the squares (cells) in 
*   the game and resets the number of iterations to zero.
* @author Victoria Lemay
* @version 1.0
*/
		private class ClearListener implements ActionListener
		{	
			
			public void actionPerformed(ActionEvent event)
			{
				for(int i=0;i<400;i++)
				{
				squares[i].setState(false);
				}
				iterationNumber=0;
				iterationLabel.setText(" Iteration Number:  "+Integer.toString(iterationNumber));
				repaint();

			}
		}
		

/**
* TimerListener is an internal class that listens to the
*   game timer. Called when the program is running 
*   automatically.
* @author Victoria Lemay
* @version 1.0
*/
		private class TimerListener implements ActionListener
		{	
			
			public void actionPerformed(ActionEvent event)
			{
				gameCellModel.initializeModel(squares);
			 	Square[] squaresAfterStep=gameCellModel.step();
				for(int i=0;i<400;i++)
				{
				squares[i].setState(squaresAfterStep[i].getState());
				}
				iterationNumber+=1;
				iterationLabel.setText(" Iteration Number:  "+Integer.toString(iterationNumber));
				repaint();

			}
		}

/**
* StartStopListener is an internal class that listens to
*   the start/stop button.
* @author Victoria Lemay
* @version 1.0
*/
		private class StartStopListener implements ActionListener
		{	
			
			public void actionPerformed(ActionEvent event)
			{
				if(intervalTimer.isRunning())
				intervalTimer.stop();
				else
				intervalTimer.start();

			}
		}


/**
* IntervalListener is an internal class to listen text 
*   field which allows users to input the delay interval.
* @author Victoria Lemay
* @version 1.0
*/
		private class IntervalListener implements ActionListener
		{	
			
			public void actionPerformed(ActionEvent event)
			{
				String intervalString=interval.getText();
				int intervalValue = Integer.valueOf(intervalString);
				intervalStatusLabel.setText(intervalString);
				if(intervalTimer.isRunning())
				intervalStatusLabel.setText("Can't change value while game is running");
				else
				{intervalStatusLabel.setText(" The current interval is: "+intervalString);
				intervalTimer.setDelay(intervalValue);
				}				
				

			}
		}

					
	
}